--[[
   Copyright 2011 Kelly Riley (lunarwater)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

]]

import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Compendium.Common.Utils";
import "Compendium.Common.UI";
import "Compendium.Common.Resources.Bundle";
local rsrc = {};

AdvPaginationControl = class( Compendium.Common.UI.PaginationControl );
function AdvPaginationControl:Constructor( cursor )
    local fontColor=Turbine.UI.Color(1,.9,.5);
    local fontFace=Turbine.UI.Lotro.Font.Verdana14;
   	local backColor=Turbine.UI.Color(.05,.05,.05);
   	local page = Turbine.UI.Lotro.TextBox();
   	
	page:SetSize(40,16);
	page:SetVisible(true);
    page:SetBackColor(Turbine.UI.Color(.25,.35,.55));
    page:SetMultiline(false);
    page:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    page:SetFont(Turbine.UI.Lotro.Font.TrajanPro13);
    page:SetForeColor(fontColor);
    page:SetOutlineColor(Turbine.UI.Color(0,0,0));
    --page:SetFontStyle(Turbine.UI.FontStyle.Outline);    
    page:SetWantsUpdates(false);
    page.FocusGained=function()
	   page:SetWantsUpdates(true);
	end
	page.FocusLost=function()
		page:SetWantsUpdates(false);
	end
   	page.Update = function(s,a) 
   		if page.Text ~= page:GetText() then
	   		page.Text = page:GetText();
	   		if self.cursor ~= nil then
	   			local success,pageNum = pcall(tonumber,page:GetText());
	   			if success and pageNum ~= nil then
	   				pageNum = self.cursor:SetPage(pageNum);
					if self.PageChanged ~= nil then
						self.PageChanged(self, 'page', self.cursor:CurPage());
					end 
	   				self:UpdatePagination();
	   			end
	   		end
	   	end
   	end
   	self.page = page;

    Compendium.Common.UI.PaginationControl.Constructor( self, cursor );

   	local jumpBack = function(sender,args)
   		self:JumpBack();  	
   	end
	local jumpForward = function(sender,args)
   		self:JumpForward();
   	end
    
    local jumpBackBtn = Turbine.UI.Lotro.Button();
    jumpBackBtn:SetParent(self);
    jumpBackBtn:SetText('« 10');
    jumpBackBtn:SetVisible(false);
   	jumpBackBtn:SetSize(55,20);
   	jumpBackBtn:SetPosition(self.prevBtn:GetLeft() + self.prevBtn:GetWidth(),0);
   	jumpBackBtn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
   	jumpBackBtn.Click = jumpBack;
	self.jumpBackBtn = jumpBackBtn;
   	
    local firstBtn = Turbine.UI.Lotro.Button();
    firstBtn:SetParent(self);
    firstBtn:SetText('First');
    firstBtn:SetVisible(false);
   	firstBtn:SetSize(55,20);
   	firstBtn:SetPosition(jumpBackBtn:GetLeft() + jumpBackBtn:GetWidth(),0);
   	firstBtn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
   	firstBtn.Click = function() self:FirstPage() end;
	self.firstBtn = firstBtn;   	
   	
    local jumpForwardBtn = Turbine.UI.Lotro.Button();
    jumpForwardBtn:SetParent(self);
    jumpForwardBtn:SetText('10 »');
    jumpForwardBtn:SetVisible(false);
   	jumpForwardBtn:SetSize(55,20);
   	jumpForwardBtn:SetPosition(self.nextBtn:GetLeft() - jumpForwardBtn:GetWidth() ,0);   
   	jumpForwardBtn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
   	jumpForwardBtn.Click = jumpForward;
   	self.jumpForwardBtn = jumpForwardBtn;
   	
    local lastBtn = Turbine.UI.Lotro.Button();
    lastBtn:SetParent(self);
    lastBtn:SetText('Last');
    lastBtn:SetVisible(false);
   	lastBtn:SetSize(55,20);
   	lastBtn:SetPosition(jumpForwardBtn:GetLeft() - lastBtn:GetWidth() ,0);   
   	lastBtn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
   	lastBtn.Click = function() self:LastPage() end;
   	self.lastBtn = lastBtn;   	


    local center = Turbine.UI.Control();
    center:SetSize(320, 20);
    center:SetBackColor(Turbine.UI.Color(0,0,0));
   	center:SetParent(self);
    center:SetPosition((self:GetWidth() / 2) - (center:GetWidth() / 2), 0);
   	center:SetZOrder(self.prevBtn:GetZOrder() - 1);
   	
    local jumpLbl = Turbine.UI.Label();
    jumpLbl:SetSize(70, 20);
    jumpLbl:SetParent(center);
    jumpLbl:SetPosition(0,0);
    jumpLbl:SetText('Go to Id:');
    jumpLbl:SetBackColor(backColor);
    jumpLbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    jumpLbl:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
    jumpLbl:SetSelectable(true);
    jumpLbl:SetForeColor(fontColor);
   	local jumpTo = Turbine.UI.Lotro.TextBox();
	jumpTo:SetParent(center);
   	jumpTo:SetPosition(jumpLbl:GetWidth(),2);   	
	jumpTo:SetSize(80,16);
	jumpTo:SetVisible(true);
    jumpTo:SetBackColor(Turbine.UI.Color(.25,.35,.55));
    jumpTo:SetMultiline(false);
    jumpTo:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    jumpTo:SetFont(Turbine.UI.Lotro.Font.TrajanPro13);
    jumpTo:SetForeColor(fontColor);
    jumpTo:SetOutlineColor(Turbine.UI.Color(0,0,0));
    --jumpTo:SetFontStyle(Turbine.UI.FontStyle.Outline);    pageNum
    jumpTo:SetWantsUpdates(false);
    jumpTo.FocusGained=function()
	   jumpTo:SetWantsUpdates(true);
	end
	jumpTo.FocusLost=function()
		jumpTo:SetWantsUpdates(false);
	end
   	jumpTo.Update = function(s,a) 
   		if jumpTo.Text ~= jumpTo:GetText() then
	   		jumpTo.Text = jumpTo:GetText();
	   		if self.cursor ~= nil then
	   			local success,itemId = pcall(tonumber,jumpTo:GetText());
	   			if success and itemId ~= nil then
	   				local pageNum = self:SearchForId(itemId);
	   				if pageNum == nil then return end;
	   				
	   				pageNum = self.cursor:SetPage(pageNum);
					if self.PageChanged ~= nil then
						self.PageChanged(self, 'page', self.cursor:CurPage());
					end 
	   				self:UpdatePagination();
	   			end
	   		end
	   	end
   	end
   	self.jumpTo = jumpTo;
    local pageLbl = Turbine.UI.Label();
    pageLbl:SetSize(170, 20);
    pageLbl:SetParent(center);
    pageLbl:SetPosition(jumpTo:GetLeft()+jumpTo:GetWidth(),0);
    pageLbl:SetText('');
    pageLbl:SetBackColor(backColor);
    pageLbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    pageLbl:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
    pageLbl:SetSelectable(true);
    pageLbl:SetForeColor(fontColor);
   	self.pageLbl = pageLbl;
   	
   	self.SizeChanged = function (s,a) 
   		local width = self:GetWidth();
		self.nextBtn:SetLeft(width - self.nextBtn:GetWidth());
		jumpForwardBtn:SetLeft(self.nextBtn:GetLeft() - jumpForwardBtn:GetWidth());   
		lastBtn:SetLeft(jumpForwardBtn:GetLeft() - lastBtn:GetWidth());
		center:SetPosition((self:GetWidth() / 2) - (center:GetWidth() / 2), 0);   
   	end

	page:SetParent(self);
   	page:SetPosition(self.prevBtn:GetLeft(),2);

   	self:UpdatePagination();
   	
end

function AdvPaginationControl:SearchForId(itemId)
	for i, rec in pairs(self.cursor.data) do
		if rec.id == itemId then
			return math.ceil(i / self.cursor.pagesize);
		end
	end
	return nil;
end

function AdvPaginationControl:JumpBack()
	if self.cursor ~= nil then
	 	local pageNum = self.cursor:CurPageNum();
 		pageNum = pageNum - 10;
		pageNum = self.cursor:SetPage(pageNum);
		if self.PageChanged ~= nil then
			self.PageChanged(self, 'page', self.cursor:CurPage());
		end 
		self:UpdatePagination();
	end
end
function AdvPaginationControl:JumpForward()
	if self.cursor ~= nil then
	 	local pageNum = self.cursor:CurPageNum();
 		pageNum = pageNum + 10;
		pageNum = self.cursor:SetPage(pageNum);
		if self.PageChanged ~= nil then
			self.PageChanged(self, 'page', self.cursor:CurPage());
		end 
		self:UpdatePagination();
	end
end

function AdvPaginationControl:FirstPage()
	if self.cursor ~= nil then
	 	local pageNum = 1;
		pageNum = self.cursor:SetPage(pageNum);
		if self.PageChanged ~= nil then
			self.PageChanged(self, 'page', self.cursor:CurPage());
		end 
		self:UpdatePagination();
	end
end
function AdvPaginationControl:LastPage()
	if self.cursor ~= nil then
	 	local pageNum = self.cursor:PageCount();
		pageNum = self.cursor:SetPage(pageNum);
		if self.PageChanged ~= nil then
			self.PageChanged(self, 'page', self.cursor:CurPage());
		end 
		self:UpdatePagination();
	end
end


function AdvPaginationControl:UpdatePagination()
	-- check if initialized all the way yet
	if self.jumpBackBtn == nil then return end;
	
	self.prevBtn:SetVisible(false);
	self.nextBtn:SetVisible(false);
	self.jumpBackBtn:SetVisible(false);
	self.jumpForwardBtn:SetVisible(false);	
	self.firstBtn:SetVisible(false);
	self.lastBtn:SetVisible(false);		
	self:SetText('');
	if self.cursor ~= nil then
		self.page:SetVisible(true);
		local pageNum = self.cursor:CurPageNum();
		if pageNum ~= self.page:GetText() then self.page:SetText(pageNum) end;
		self.pageLbl:SetText(self.cursor:tostring()); -- .. ' ' .. self.paginationText);
		if self.cursor:HasPrev() then 
			self.prevBtn:SetVisible(true) 
			self.jumpBackBtn:SetVisible(true);
			self.firstBtn:SetVisible(true);
		end;
		if self.cursor:HasNext() then 
			self.nextBtn:SetVisible(true) 
			self.jumpForwardBtn:SetVisible(true);	
			self.lastBtn:SetVisible(true);				
		end;
		if self.prevBtn:IsVisible() then
			self.page:SetLeft(self.firstBtn:GetLeft() + self.firstBtn:GetWidth());
		else
			self.page:SetLeft(self.prevBtn:GetLeft());
		end
	else
		self.page:SetVisible(false);
	end
end
